#! /bin/csh -f
#
#	Combinatorial EPMR Molecular-Replacement script      James Holton 8-2-04
#
# EPMR is available from Agouron Pharmaceuticals, Inc.
# ftp://ftp.agouron.com/pub/epmr/
#
alias nawk nawk
nawk 'BEGIN{exit}' >& /dev/null
if($status) alias nawk awk
####################################
# defaults for EPMR program
set nmols = ""
set RESOs = ""
set SGs   = ""

set pdbfiles = ""
set mtzfile  = mtz/all.mtz
set F        = FP
set tempfile = epmr_temp

if("$1" == "") goto Help
goto Setup
####################################
Help:
cat << EOF

usage: epmr.com pdbfile.pdb [file1.pdb ...] mtzfile.mtz P212121 4-15A 1

where:
pdbfile.pdb	- anything with Brookhaven "ATOM" or "HETATM" entries in it
file1.pdb	- one or more other models you want to try
mtzfile.mtz	- CCP4 "mtz" format data file
P212121		- space group to search
4-15A		- resolution range (4A to 15A)
1		- number of molecules to place (integer)

note:
you can specify more than one space group, more than one model, and 
more than one resolution range, they will be run combinatorially.
EOF
exit 9
####################################
Return_from_Setup:

echo "data:        $F in $mtzfile"
echo "models:      $pdbfiles"
echo "resolution:  $RESOs"
echo "chains:      $nmols"
echo "space group: $SGs"
echo ""
echo "model SG n reso" | nawk -v format="$format" '{printf format, $1, $2, $3, $4}'
echo ""


# try each model, in turn
foreach pdbfile ( $pdbfiles ) 

# prefix for output log/file
set prefix = `basename $pdbfile .pdb`
set prefix = `basename $prefix .brk`

foreach RESO ( $RESOs )

# update resolution limits
set temp = `echo "$RESO" | nawk 'BEGIN{FS="-"} $1+0 > 0.1{print $1+0} $2+0 > 0.1{print $2+0}'`
if($#temp != 1) then
    # setting both resolution limits
    set temp = `echo $temp | nawk '$1>$2{print $1, $2} $2>$1{print $2, $1}'`
    if($#temp == 2) then
	set loRES = "$temp[1]"
	set hiRES = "$temp[2]"
    endif
else
    # only setting high-resolution limit
    if("$temp" != "") set hiRES = "$temp"
endif

foreach nmol ( $nmols )

foreach SG ( $SGs )

# check for orthorhombic "pseudo-spacegroups"
set axes = ""
set realSG = "$SG"
if("$SG" == P222)    set axes = "a b c"
if("$SG" == P212121) set axes = "a b c"
if("$SG" == P2221) then
    # P2221 with screw along longest axis
    set axes  = "a b c"
    set realSG = "P2221"
endif
if("$SG" == P2212) then
    # P2221 with screw along mid-length axis
    set axes  = "b c a"
    set realSG = "P2221"
endif
if("$SG" == P2122) then
    # P2221 with screw along shortest axis
    set axes  = "c a b"
    set realSG = "P2221"
endif

if("$SG" == P21212) then
    # P21212 with non-screw along longest axis
    set axes  = "a b c"
    set realSG = "P21212"
endif
if("$SG" == P21221) then
    # P21212 with non-screw along mid-length axis
    set axes  = "b c a"
    set realSG = "P21212"
endif
if("$SG" == P22121) then
    # P21212 with non-screw along shotest axis
    set axes  = "c a b"
    set realSG = "P21212"
endif
set realSGnum = `nawk -v SG=$realSG '$4==SG{print $1;exit}' ${CLIBD}/symop.lib`

if("$axes" != "") then
    # we have an asymmetric orthorhombic space group 

    # get current axis ordering, then
    # find out what the cannonical one would be
    # then decide how to go from current ordering to the desired one
    echo "head" |\
    mtzdump hklin $mtzfile |\
    nawk '/Cell Dimensions/{getline;getline;print}' |\
    nawk '{\
	# print out current axis order \
	print $1, "h"; print $2, "k"; print $3, "l"}' |\
    sort -n |\
    nawk '\
	# add cannonical axis names\
	NR==1{print $0, "a"} NR==2{print $0, "b"} NR==3{print $0, "c"}' |\
    nawk -v axes="$axes" 'BEGIN{split(axes, abc)} {\
	# write desired axis ordering in front of cannonical one \
	print abc[NR], $0}' |\
    sort |\
    nawk '# print out new hkl order \
	  {printf "%s ", $3} END{print ""}' |\
    nawk '$1 $2 $3 $1 $2 $3 !~ /hkl/{$3 = "-" $3} {print "reindex", $1",",$2",",$3}' |\
    reindex HKLIN $mtzfile HKLOUT ${tempfile}.mtz >> /dev/null

    # this should give us a mapping between any two orthorhombics

    # now simply re-name the space group in the header
    echo "SYMM $realSGnum" |\
    mtzutils HKLIN ${tempfile}.mtz HKLOUT ${tempfile}newSG.mtz >> /dev/null
    if($status) then
	set BAD
	echo "failed! ?"
	continue
    endif
    mv ${tempfile}newSG.mtz ${tempfile}.mtz >& /dev/null

else
    # just re-name the space group in the mtz header
    echo "SYMM $realSGnum" |\
    mtzutils HKLIN $mtzfile HKLOUT ${tempfile}.mtz >> /dev/null
    if($status) then
	set BAD
	echo "failed! ?"
	continue
    endif
endif


##########################################
# convert mtz data to EPMR-readable format (only needs F, not SIGF)
rm -f ${tempfile}.hkl >& /dev/null
mtz2various hklin ${tempfile}.mtz hklout ${tempfile}.hkl << EOF >! ${tempfile}.log
OUTPUT USER '(3I5, 1F12.3)'
LABIN FP=$F
EOF
echo "head" | mtzdump hklin ${tempfile}.mtz >! ${tempfile}.log
rm -f ${tempfile}.mtz

# create "cell" file
set CELL  = `nawk '/Cell Dimensions/{getline;getline;print}' ${tempfile}.log`
set SGnum = `nawk -v SG=$realSG '$4 == SG && $1 < 500 {print $1}' $CLIBD/symop.lib | head -1`
echo "$CELL $SGnum" >! ${tempfile}.cel
rm -f ${tempfile}.log


echo "$pdbfile $SG $nmol $hiRES $loRES" |\
nawk -v format="$format" '{printf format, $1, $2, $3, $4+0 "-" $5+0 "A"}'
##########################################
# run epmr
epmr -m$nmol -h$hiRES -l$loRES -e1 ${tempfile}.cel $pdbfile ${tempfile}.hkl >! ${prefix}_${nmol}_${SG}_{$RESO}.log
# other epmr options:
# -p300  population size
# -g50   generations
# -n10   total runs
# -t0.45 acceptable correlation coefficient (threshold)
# -b0.0  minimum "bump" distance (penalize overlapping molecules)
# -w1	 print out top "1" solutions
# -s file.pdb	input a "static" structure as a partial solution
# -e0	 seed random number generator (0 means seed with clock)
##########################################

# finish off summary line
cat ${prefix}_${nmol}_${SG}_${RESO}.log |\
nawk '/^Soln/{print "  best CC=", $(NF-2), "R=", $NF}' |\
sort -n +2 | tail -1 | tee ${tempfile}stats
set R  = `nawk '{print $5}' ${tempfile}stats` 
set CC = `nawk '{print $3}' ${tempfile}stats`
rm -f ${tempfile}stats >& /dev/null

# record what the stats were in the PDB file
echo "REMARK EPMR result: $pdbfile $SG $nmol ${hiRES}-${loRES}A    best CC= $CC R= $R" |\
cat >! ${prefix}_${nmol}_${SG}_{$RESO}.best.pdb

# add a header to the PDB file
echo "" >! ${tempfile}.pdb
echo "CELL $CELL" |\
pdbset XYZIN ${tempfile}.pdb XYZOUT ${tempfile}cell.pdb > /dev/null
head -4 ${tempfile}cell.pdb >> ${prefix}_${nmol}_${SG}_{$RESO}.best.pdb
rm -f ${tempfile}.pdb ${tempfile}cell.pdb >& /dev/null

set i = 0
while( $i < $nmol )
    @ i = ( $i + 1 )
    # re-name output files
    if((! -e epmr.${i}.best.pdb) && (-e epmr.${i}.pdb)) mv epmr.${i}.pdb epmr.${i}.best.pdb

    cat epmr.${i}.best.pdb >> ${prefix}_${nmol}_${SG}_{$RESO}.best.pdb
    rm -f epmr.${i}.best.pdb >& /dev/null
end

end

end
end
end

#clean up
rm -f ${tempfile}.hkl ${tempfile}.cel >& /dev/null

exit

##########################################
##########################################
##########################################
##########################################
Setup:
# read some command-line options
set hiRES = 4
set loRES = 15

# scan the command line for files
set i = 0
while( $i < $#argv )
    @ i = ( $i + 1 )
    @ nexti = ( $i + 1 )
    @ lasti = ( $i - 1 )
    if($nexti > $#argv) set nexti = $#argv
    if($lasti < 1) set lasti = 1
    set arg = "$argv[$i]"

    # check for mtz file
    if( "$arg" =~ *.mtz ) then
	if(! -e "$arg") then
	    echo "WARNING: $arg does not exist! "
	    continue
	endif
	set mtzfile  = "$arg"
	continue
    endif
    if(("$arg" =~ *.pdb)||("$arg" =~ *.brk)) then
	if(! -e "$arg") then
	    echo "WARNING: $arg does not exist! "
	    continue
	endif
	set pdbfiles = ( $pdbfiles $arg )
	continue
    endif
    # check for space groups
    if("$arg" =~ [PpCcIiFfRr][1-6]*) then
	set temp = `echo $arg | nawk '{print toupper($1)}'`
	set temp = `nawk -v SG=$temp '$4 == SG {print $4}' $CLIBD/symop.lib | head -1`
	if("$temp" == "") then
	    # check for "pseudo-spacegroup" language
	    set temp = `echo "$arg" | nawk 'toupper($1) ~ /[PC]2212|[PC]2122|P21221|P22121/'`
	    # these are okay too, reindexing engine will understand
	endif
	if("$temp" != "") then
	    # add this SG to the space group list
	    set SGs = ( $SGs $temp )
	    continue
	endif
    endif
    if("$arg" =~ [0-9]*) then
	# we have a number
	if(("$arg" =~ *A)||("$argv[$nexti]" == "A")) then
	    # user-preferred resolution limits
	    set temp = `echo "$arg" | nawk 'BEGIN{FS="-"} $1+0 > 0.1{print $1+0} $2+0 > 0.1{print $2+0}'`
	    if($#temp != 1) then
		set temp = `echo $temp | nawk '$1>$2{print $1, $2} $2>$1{print $2, $1}'`
		if($#temp == 2) then
		    set RESOs = "$RESOs $arg"
		endif
	    else
		if("$temp" != "") set RESOs = "$RESOs $arg"
	    endif
	endif
	
	# interpret free integers as number of molecules?
	set int = `echo "$arg" | nawk '{printf "%d", $1+0}'`
	if("$arg" == "$int") then
	    set nmols = ( $nmols $int )
	endif
    endif
end

# go no further if there is no mtz file
if(! -e "$mtzfile") goto Help

#####################################
#get dataset names from mtz file
echo "go" | mtzdump hklin $mtzfile |\
cat >! ${tempfile}mtzdump

# get info from the MTZ 
set SG    = `nawk '/Space group/{print $5}' ${tempfile}mtzdump`
set SGnum = `nawk '/Space group/{print $NF+0}' ${tempfile}mtzdump `
set SG    = `nawk -F "[\047]" '/Space group/{print $2}' ${tempfile}mtzdump `
set SG    = `nawk -v num=$SGnum '$1==num && NF>5{print $4}' ${CLIBD}/symop.lib `
set PG    = `nawk -v num=$SGnum '$1==num && NF>5{print $5}' $CLIBD/symop.lib | head -1`
set latt = `echo $SG | nawk '{print substr($1, 1, 1)}'`
set allSGs = `nawk -v PG=$PG -v latt=$latt '$5==PG && $4 ~ latt && ! /m/ && ! /bar/ && $1 < 500{print $4}' $CLIBD/symop.lib`

# special case of asymmetric orhorhombics
if("$PG" == "PG222") then
    if("$latt" == "P") set allSGs = "P212121 P21212 P21221 P22121 P2221 P2212 P2122 P222"
endif
if("$SGs" == "") set SGs = "$allSGs"
# epmr default:
if("$RESOs" == "") set RESOs = "4-15A"
if("$nmols" == "") set nmols = "1"

# use completeness, or F/sigF to pick default F
cat ${tempfile}mtzdump |\
nawk '/OVERALL FILE STATISTICS/,/No. of reflections used/' |\
nawk 'NF>10 && $(NF-1) ~ /[FQPWADI]/' |\
nawk '$(NF-1) == "F"{F=$NF; meanF=$8; reso=$(NF-2); comp=substr($0,32)+0; \
      getline; S=$NF; if($8) meanF /= $8; print F, S, reso, comp, meanF;}' |\
sort +2n -3 +3nr -4 +4nr >! ${tempfile}F

# and extract all dataset types/labels
cat ${tempfile}mtzdump |\
nawk '/OVERALL FILE STATISTICS/,/No. of reflections used/' |\
nawk 'NF>10 && $(NF-1) ~ /[FQPWADI]/' |\
nawk 'NF>2{print $(NF-1), $NF, " "}' |\
cat >! ${tempfile}cards

#clean up a bit
rm -f ${tempfile}mtzdump

#####################################
# pick F with best resolution, or <F>/<sigma(F)>
set F    = `head -1 ${tempfile}F`
if($#F > 2) then
    set SIGF = $F[2]
    set F    = $F[1]
endif

# see if user specified an F
foreach arg ( $* )
    set temp = `grep " $arg " ${tempfile}cards`
    if("$temp" =~ F*) then
	set F = "${arg}"
	set temp = `nawk -v arg="$arg" '$1==arg{print $2}' ${tempfile}F`
	if($#temp == 1) set SIGF = "$temp"
	continue
    endif
end
rm -f ${tempfile}cards ${tempfile}F >& /dev/null

if("$F" == "") goto Help

# devise a clean printing-out format
set w1 = `echo "$pdbfiles" | nawk '{for(i=1;i<=NF;++i) print length($i)}' | sort -n | tail -1`
set w2 = `echo "$SGs" | nawk '{for(i=1;i<=NF;++i) print length($i)}' | sort -n | tail -1`
set w3 = `echo "$RESOs" | nawk '{for(i=1;i<=NF;++i) print length($i)}' | sort -n | tail -1`
set format = "%-${w1}s %${w2}s %s %${w3}s"

goto Return_from_Setup

exit
####################
# the future
combinatorial B-factor, mtz
