#! /bin/csh -f
#
#
#	shelx.com	- combinatorial shelx-based heavy-atom finder  James Holton 12-2-00
#
#   will try all possible combinations of an arbitrary number of:
#   1) space groups
#   2) data sets (multiple mtz files are okay)
#   3) resolution cutoffs
#   4) F/sigma cutoffs
#   5) maximum delta-F cutoffs (default to a reasonable value)
#   6) expected site counts
#
#   the results of each shelx run are also passed through the CCP4 "vecref"
#   program, using a Patterson calculated with the same data presented to
#   shelx.  Sites that don't agree with the Patterson are rejected by vecref.
#
#   a pdb file ("outfile" below) is produced for each combinatorial run
#
#   "good" shelx sites are in chain:   D
#   "maybe" shelx sites are in chain:  Q
#   sites on special positons are in:  X
#
#   however, this pdb file will be overwritten if the peak heights look
#   "better" than the last "best" soluton.
#
#   the output lines that this script prints to the terminal can each be
#   used as command-line input to re-run the same procedure, updating the
#   output file.
#
# location of shelxs executable
set SHELX = /programs/bin/shelxs
set prefix = "shelx"

# location of awk
alias nawk nawk
nawk 'BEGIN{print}' >& /dev/null
if($status) alias nawk awk

set mtzfile  = ./mtz/all.mtz
set outfile  = shelx.pdb
set logfile  = ./shelx.log
set tempfile = ./shelx_temp

# crystal variables
set hiRES = ""
set loRES = 1000
set CELL  = ""
set SG    = ""
set Ee    = "Se"

# trials
set SGs           = ""
set RESOs         = ""
set target_sites  = "1 3 5 10 15 20"
set sigma_cutoffs = "0 1 2 3"
set max_diffs     = "auto"

# no args? need help
if($#argv == 0) goto Help

# this procedure (re)sets most of the above variables
# from either the provided files, or the command line
goto Setup

Help:
cat << EOF

usage: $0 data.hkl 6 sites P212121

where:
data.mtz	- contains the F, Fs, or Dano you want to run shelx on
6 sites		- your expected number of sites
2A		- your desired resolution cutoff
1sig		- your desired F/sigma cutoff
diff 50		- your desired maximum-difference cutoff
P212121		- space group (optional)

$0 will run shelxs (direct phasing) on the data provided in the
space group(s) given.

EOF

exit
#
#   This procedure (at the bottom of the script) does the following
#   1) scan the command line for the mtz file(s)
#   2) set the CELL, SG, and other variables
#   3) generate dataset list file: ${tempfile}datasets
Return_from_Setup:

################################################################################
# back-up values (shouldn't be empty)
if("$SGs" == "")           set SGs           = "$SG"
if("$RESOs" == "")         set RESOs         = `echo "$hiRES 0.1 4" | nawk '$3-1>0{v=1;vstep=(1-$2)/($3-1); for(i=1;i<=$3;++i){printf "%.1f ", (v*1/($1^3))^(-1/3);v-=vstep;}}'`
if("$RESOs" == "")         set RESOs         = "$hiRES"
if("$sigma_cutoffs" == "") set sigma_cotoffs = "1"
if("$target_sites" == "")  set target_sites  = "1"
if("$max_diffs" == "")     set max_diffs     = "auto"

# print intended combinatorial
cat << EOF | tee $logfile
          SGs: $SGs
  reso limits: $RESOs
sigma cutoffs: $sigma_cutoffs
 target sites: $target_sites
    max diffs: $max_diffs
EOF
echo -n "    data sets: " | tee -a $logfile
cat ${tempfile}datasets |\
nawk 'NF==4{print $2,"from",$4} NF>4{print $2 "-" $5,"from",$4,$7} {printf "               "} END{print ""}' |\
tee -a $logfile

echo "running SHELX:            ---- cutoffs ------           shelx results   Patterson-check"
echo "spacegroup data set       reso  sigma maximum    expect found    CFOM  survivors  avg  max  mtz file(s)"


################################################################################
# explore all indicated possibilities
foreach SG ( $SGs )
    # retrieve ASU count in this space group
    set ASU_per_CELL = `nawk -v SG=$SG '$4==SG{print $2}' $CLIBD/symop.lib | head -1`
    
    
foreach dataset ( `nawk '{print NR}' ${tempfile}datasets` )

    # extract labels from the "dataset" file
    set F1    = `nawk -v dataset=$dataset 'NR==dataset{print $2}' ${tempfile}datasets`
    set SIGF1 = `nawk -v dataset=$dataset 'NR==dataset{print $3}' ${tempfile}datasets`
    set mtz1  = `nawk -v dataset=$dataset 'NR==dataset{print $4}' ${tempfile}datasets`
    
    set F2    = `nawk -v dataset=$dataset 'NR==dataset{print $5}' ${tempfile}datasets`
    set SIGF2 = `nawk -v dataset=$dataset 'NR==dataset{print $6}' ${tempfile}datasets`
    set mtz2  = `nawk -v dataset=$dataset 'NR==dataset{print $7}' ${tempfile}datasets`
        
    # (F or D)
    set type  = `nawk -v dataset=$dataset 'NR==dataset{print $1}' ${tempfile}datasets`
    
    if("$F2" == "") then
	set F2 = "$F1"
	set SIGF2 = "$SIGF1"
	set mtz2  = "$mtz1"
	unset SUBTRACT
    else
	# need to do the subtraction
	set SUBTRACT
    endif
    
    # check for orthorhombic "pseudo-spacegroups"
    set axes = ""
    set realSG = "$SG"
    if("$SG" == P222)    set axes = "a b c"
    if("$SG" == P212121) set axes = "a b c"
    if("$SG" == P2221) then
	# P2221 with screw along longest axis
	set axes  = "a b c"
	set realSG = "P2221"
    endif
    if("$SG" == P2212) then
	# P2221 with screw along mid-length axis
	set axes  = "b c a"
	set realSG = "P2221"
    endif
    if("$SG" == P2122) then
	# P2221 with screw along shortest axis
	set axes  = "c a b"
	set realSG = "P2221"
    endif
    
    if("$SG" == P21212) then
	# P21212 with non-screw along longest axis
	set axes  = "a b c"
	set realSG = "P21212"
    endif
    if("$SG" == P21221) then
	# P21212 with non-screw along mid-length axis
	set axes  = "b c a"
	set realSG = "P21212"
    endif
    if("$SG" == P22121) then
	# P21212 with non-screw along shotest axis
	set axes  = "c a b"
	set realSG = "P21212"
    endif
    set realSGnum = `nawk -v SG=$realSG '$4==SG{print $1;exit}' ${CLIBD}/symop.lib`
    
    if("$axes" != "") then
	# we have an asymmetric orthorhombic space group 
	set i = 0
	foreach mtz ( $mtz1 $mtz2 )
	    @ i = ( $i + 1 )
	    # get current axis ordering, then
	    # find out what the cannonical one would be
	    # then decide how to go from current ordering to the desired one
	    echo "head" |\
	    mtzdump hklin $mtz |\
	    nawk '/Cell Dimensions/{getline;getline;print}' |\
	    nawk '{\
		# print out current axis order \
		print $1, "h"; print $2, "k"; print $3, "l"}' |\
	    sort -n |\
	    nawk '\
		# add cannonical axis names\
		NR==1{print $0, "a"} NR==2{print $0, "b"} NR==3{print $0, "c"}' |\
	    nawk -v axes="$axes" 'BEGIN{split(axes, abc)} {\
		# write desired axis ordering in front of cannonical one \
		print abc[NR], $0}' |\
	    sort |\
	    nawk '# print out new hkl order \
	          {printf "%s ", $3} END{print ""}' |\
	    nawk '$1 $2 $3 $1 $2 $3 !~ /hkl/{$3 = "-" $3} {print "reindex", $1",",$2",",$3}' |\
	    reindex HKLIN $mtz HKLOUT ${tempfile}.mtz >> $logfile

	    # this should give us a mapping between any two orthorhombics

	    # now simply re-name the space group in the header
	    echo "SYMM $realSGnum" |\
	    mtzutils HKLIN ${tempfile}.mtz HKLOUT ${tempfile}mtz${i}.mtz >> $logfile
	    if($status) then
		set BAD
		goto Clean_up
	    endif
	    rm -f ${tempfile}.mtz >& /dev/null
	end
    else
	# just re-name the space group in each mtz header
	echo "SYMM $realSGnum" |\
	mtzutils HKLIN $mtz1 HKLOUT ${tempfile}mtz1.mtz >> $logfile
	if($status) then
	    set BAD
	    goto Clean_up
	endif
	echo "SYMM $realSGnum" |\
	mtzutils HKLIN $mtz2 HKLOUT ${tempfile}mtz2.mtz >> $logfile
	if($status) then
	    set BAD
	    goto Clean_up
	endif
    endif
    

    # extract the two datasets into a single file
    cad hklin1 ${tempfile}mtz1.mtz hklin2 ${tempfile}mtz2.mtz \
        hklout ${tempfile}cadded.mtz << EOF-cad >> $logfile
    LABIN FILE 1 E1=$F1  E2=$SIGF1
    CTYPI FILE 1 E1=F    E2=Q
    LABOU FILE 1 E1=F1   E2=SIGF1
    LABIN FILE 2 E1=$F2  E2=$SIGF2
    CTYPI FILE 2 E1=F    E2=Q
    LABOU FILE 2 E1=F2   E2=SIGF2
EOF-cad
    
    # make sure we have the proper cell
    set CELL  = `echo "head" | mtzdump hklin ${tempfile}cadded.mtz | nawk '/Cell Dimensions/{getline;getline;print}'`
    
    # come up with a reasonable protein size (assume Vm=2.5)
    echo $CELL |\
    nawk 'NF==6{s=3.1415926535897899419/180; A=cos(s*$4); B=cos(s*$5); G=cos(s*$6); \
     skew = 1 + 2*A*B*G - A*A - B*B - G*G ; if(skew < 0) skew = -skew;\
     printf "%.3f\n", $1*$2*$3*sqrt(skew)}' |\
    cat >! ${tempfile}volume
    set CELLvolume = `cat ${tempfile}volume`
    rm -f ${tempfile}volume >> /dev/null
        
    # estimate no of protein atoms in cell assume Vm=2.5 and protein atom = 14 amu
    set protein = `echo "$CELLvolume $ASU_per_CELL" | nawk '$2+0>0{printf "%d", ((($1+1)/2.5)/14)/$2}'`
    if("$protein" == "") set protein = 2000
    #echo "we estimate $protein protein atoms in ASU."


    # iterate over resolution cutoffs
    foreach cutRES ( $RESOs )
    
    # apply scaleit (to scale isomorphous differences, and check DIFF recommendation)
    if($?SUBTRACT) then
	set refine = "refine anisotropic"
	set scale  = ""
    else
	# either Dano or FH data (squash F2)
	set refine = "analyze"
	set scale  = "SCALE FPH1 0.000001"
    endif
    scaleit hklin ${tempfile}cadded.mtz \
            hklout ${tempfile}scaled.mtz << EOF-scale | tee ${tempfile}scaleit.log >> $logfile
    RESOLUTION 1000 $cutRES
    $refine
    $scale
    LABIN FP=F1 SIGFP=SIGF1 FPH1=F2 SIGFPH1=SIGF2
EOF-scale
    
    # iterate over sigma- and maximum-difference cutoffs
    foreach cutSIG ( $sigma_cutoffs )
    foreach diff ( $max_diffs )
    
    # use scaleit's recommendation by default
    if("$diff" == "auto") set diff = `nawk '/acceptable differences/{print $NF}' ${tempfile}scaleit.log`
    if("$diff" == "") set diff = 99999999
    
    # make a Patterson map
    set labin = "F1=F1 SIG1=SIGF1 F2=F2 SIG2=SIGF2"
    if($?SUBTRACT) then
	set scale = 1
	set fftmtz = ${tempfile}scaled.mtz
    else
	# F2 is zero (for calculating DIFF)
	set scale = 0.0000001
	set fftmtz = "${tempfile}cadded.mtz"
    endif
    fft HKLIN $fftmtz MAPOUT ${tempfile}patt.map << EOF-fft >> $logfile
    TITLE ${cutRES}A Patterson of $F1-${scale}$F2 excluding diff>$diff
    RESOLUTION $cutRES
    PATTERSON
    LABIN $labin
    SCALE F2 $scale 0
    EXCLUDE DIFF $diff
    EXCLUDE SIG1 $cutSIG
EOF-fft
    # normalize the Patterson
    echo "scale sigma" |\
    mapmask mapin  ${tempfile}patt.map \
           mapout ${tempfile}.map >> $logfile
    mv ${tempfile}.map ${tempfile}patt.map
    if($?user_patt) then
	# option to use an outside Patterson?
	cp $user_patt ${tempfile}patt.map
    endif
    
    # extract the dataset in shelx format
    set exportmtz = "$fftmtz"
    set labin = "FP=F1 SIGFP=SIGF1"
    if($?SUBTRACT) set labin = "$labin FPH=F2 SIGFPH=SIGF2"
    if("$type" == "D") then
	# make sure we treat Danos properly
	set labin = "DP=$F1 SIGDP=$SIGF1"
	set exportmtz = ${tempfile}mtz1.mtz
    endif

    
    mtz2various hklin $exportmtz hklout ${tempfile}.hkl << EOF >> $logfile
    OUTPUT SHELX
    FSQUARED
    RESOLUTION $cutRES
    #EXCLUDE SIGP $cutSIG
    #EXCLUDE SIGH $cutSIG
    EXCLUDE DIFF $diff
    LABIN $labin
EOF
    # strip off the text header
    nawk '! /[A-Z]/' ${tempfile}.hkl >! ${prefix}.hkl

    # use the output to make a shelx .ins file as well
    nawk '! /[A-Z]/{exit} {print}' ${tempfile}.hkl |\
    grep -v HKLF >! ${prefix}.ins
    rm -f ${tempfile}.hkl >& /dev/null
    
    # finish it up
    echo "SFAC N $Ee"    >> ${prefix}.ins
    echo "$protein $ASU_per_CELL" | nawk '{print "UNIT", sqrt($1),$2}' >> ${prefix}.ins
    echo "OMIT $cutSIG" >> ${prefix}.ins
    echo "TREF"         >> ${prefix}.ins
    echo "HKLF 4"       >> ${prefix}.ins
    echo "END"          >> ${prefix}.ins
    
    foreach expect ( $target_sites )
    
    # print mini-report (suitable for re-input on command line)
    if($?SUBTRACT) then
	set dataset = "${F1}-${F2}"
    else
	set dataset = "$F1"
    endif
    echo "$SG $dataset $cutRES $cutSIG $diff $expect" |\
	 nawk '{printf "%-10s %-13s %4.1fA %4.1fs  diff %-5.1f %3d sites ", $1, $2, $3, $4, $5, $6}'
    
    # shelx wants sites in CELL not ASU
    set sites_in_cell = `echo "$expect $ASU_per_CELL" | nawk '{print $1 * $2}'`
    
    # edit the shelx input file
    cat ${prefix}.ins |\
    nawk -v sites=$sites_in_cell '/^UNIT/{$3=sites} {print}' |\
    cat >! ${tempfile}.ins
    mv ${tempfile}.ins ${prefix}.ins >& /dev/null
    
    #######################################
    # run shelx
    ${SHELX} ${prefix} >> $logfile
    #######################################
    
    # check the results
    cat ${prefix}.lst |\
    nawk '/Heavy-atom assignments:/{slist=1} /optimization/{slist=0}\
	  /Peak  Atom     x/{plist=1}\
          slist && $5~/[0-9]/{++s;ssum+=$6;mins=$6}\
	  plist && $2~/^Q/{++q;qsum+=$1;if(maxq+0==0)maxq=$1}\
          END{print "SITES", s+0; \
	      if(s+0==0) s=1; savg=ssum/s;\
	      if(q+0==0) q=1; qavg=qsum/q;\
	      if(qavg==0) qavg=100; if(maxq+0==0)maxq=100;\
	      print "HEIGHT", ssum/s, mins;\
	      print "QAVG", qsum/q, maxq;\
	      print "RATIO", savg/qavg;\
	      print "DISCR", mins/maxq}' |\
    cat >! ${tempfile}.results
    cat ${prefix}.lst |\
    nawk '/with CFOM/{print "CFOM", $NF+0; exit}' |\
    cat >> ${tempfile}.results
    cat ${prefix}.lst |\
    nawk '/Try    Ralpha Nqual/,/CFOM Range/' |\
    nawk 'NF>=6 && ! /[A-Z]/{++n;sum+=$6} \
          END{if(n) print "meanCFOM", sum/n}' |\
    cat >> ${tempfile}.results
    
    set found  = `nawk '/^SITES/{print $2}' ${tempfile}.results`
    set DISCR  = `nawk '/^DISCR/{print $2}' ${tempfile}.results`
    set RATIO  = `nawk '/^RATIO/{print $2}' ${tempfile}.results`
    set bCFOM  = `nawk '/^CFOM/{print $2}' ${tempfile}.results`
    set mCFOM  = `nawk '/^meanCFOM/{print $2}' ${tempfile}.results`
    rm -f ${tempfile}.results >& /dev/null
    
    # display for user
    echo "$found $bCFOM " | nawk '{printf "%-4d  %4.2f  ", $1, $2, $3 " " $4 " " $5}'
    
    # retrieve shelx's atoms in a more managable format
    cat ${prefix}.lst |\
    nawk '/Heavy-atom assignments:/{list=1} /optimization/{list=0}\
	  /Peak  Atom     x/{plist=1}\
          list {print $NF, $0}\
	  plist && $2 ~ /^Q/{print $0, $1}' |\
    nawk 'NF==7{n=substr($2,match($2,"[1-9]"));\
                Ee=substr($2,1,match($2,"[1-9]")-1);\
		x=$3;y=$4;z=$5;occ=$6;height=$NF;\
		chain="D"; if(Ee=="Q")chain=Ee; if(occ<1)chain="X";\
    print x, y, z, occ, height/10, Ee, n, chain;}' |\
    cat >! ${tempfile}allatoms
    # format: xf yf zf 1/mult height Ee atmno chain
    
    # use vecref to check against the Patterson
    cat ${tempfile}allatoms |\
    nawk -v cutRES=$cutRES '{++n} $4 !~ /\.0/{next;$1+=0.001; $2+=0.001; $3+=0.001}\
    NF>6{print "ATOM H", n, 1, $1, $2, $3, 79*(cutRES/3)^2}' |\
    cat >! ${tempfile}atoms.out
    set orig = `cat ${tempfile}atoms.out | wc -l`
    
    # do some cycles of atom rejection
    set rcycles = "3,0,0 3,0,0 3,10,0 3,10,0 3,0,10 3,0,10"
    set rcycles = "3,0,0 3,0,0 3,0,0"
    if($?NO_PATT) set rcycles = ""
    foreach rcycle ( $rcycles )
	set rcycle = `echo "$rcycle" | nawk 'BEGIN{FS=","} {print $1, $2, $3}'`
	
	# reduce resolution a bit so vecref won't crash
	set refRES = `echo $cutRES | nawk '{print (0.8*1/($1^3))^(-1/3)}'`
	
	# make sure there aren't >50 sites
	sort -n +2 ${tempfile}atoms.out |\
	head -50 >! ${tempfile}atoms.in
	
	# refine "occupancies" against the Patterson
	rm -f ${tempfile}atoms.out >& /dev/null
	set vecrefSG = $realSG
	if("$vecrefSG" == "H3") set vecrefSG = R3
	if("$vecrefSG" == "H32") set vecrefSG = R32
	vecref mapin ${tempfile}patt.map ATOUT ${tempfile}atoms.out << EOF-vecref >>& $logfile
	SPACEGROUP $vecrefSG
	RESOLUTION ${refRES}
	CYCLES $rcycle
	BLIMIT 0 1000
	BREF
	@${tempfile}atoms.in
EOF-vecref
	if(($status)||(! -e ${tempfile}atoms.out)) then
	    # all atoms rejected
	    echo -n "" >! ${tempfile}atoms.out
	    break
	endif
    end
    rm -f ${tempfile}atoms.in >& /dev/null
    if($?NO_PATT) then
	# no data available from Patterson
	echo -n "" >! ${tempfile}atoms.out
    endif
    
    # label the peaks file with the vecref occupancies
    cat ${tempfile}atoms.out ${tempfile}allatoms |\
    nawk '/^ATOM/{occ[$3]=$4; next} {++n; print $0, occ[n]+0}' |\
    cat >! ${tempfile}atoms.pocc
    # format xf yf zf 1/mult height Ee atmno chain pocc

    rm -f ${tempfile}atoms.out >& /dev/null
    rm -f ${tempfile}allatoms >& /dev/null
    
    # print out stats
    cat ${tempfile}atoms.pocc |\
    nawk '$6~/^Q/{next} $NF+0>0{++patt}\
	  {++n; sum+=$NF} $NF>max{max=$NF}\
          END{printf "%d ", patt; if(n==0)n=1; print sum/n, max+0}' |\
    cat >! ${tempfile}.results
    if($?NO_PATT) then
	# fake it, but let user know nothing happened
	echo "? 0 0" >! ${tempfile}.results
    endif
    set patt = `nawk '{print $1}' ${tempfile}.results`
    set avg  = `nawk '{print $2}' ${tempfile}.results`
    set max  = `nawk '{print $3}' ${tempfile}.results`
    rm -f ${tempfile}.results >& /dev/null

    # print out for user
    echo "$patt $orig $avg $max" | nawk '{printf "%9s %4.1f %4.1f ", $1"/"$2, $3, $4}'
    
    # print out data files used
    set mtzfiles = "$mtz1"
    if("$mtz2" != "$mtz1") set mtzfiles = "$mtz1 $mtz2"
    echo -n " $mtzfiles "
    
    
    # come up with some kind of holistic "score"?     
    echo "$expect $found $bCFOM $patt $avg $max" |\
    nawk '{expect=$1; found=$2; CFOM=$3; patt=$4+0; avg=$5; max=$6;\
    sites=found; if(sites<1) sites=0.5; if(patt<1) patt=0.5;\
    if($4 ~ /^\?/) avg=1/CFOM;\
    printf "%s %s %s ",  1/CFOM, avg*sites*sites/patt, sites*max}' |\
    cat >! ${tempfile}.score

    # sum-of-sums score
    # format xf yf zf 1/mult height Ee atmno chain pocc
    cat ${tempfile}atoms.pocc |\
    nawk -v expect=$expect '$6~/^Q/ || n>=expect{next} {++n}\
	  $NF*$5>0{sum+=$NF+$5; sum_s+=$5}\
          END{if(n==0)n=1; print sum/n, sum_s}' |\
    cat >! ${tempfile}.score

    set score = `nawk '{printf "%d", 1000*$1}' ${tempfile}.score`
    set sum_s = `nawk '{print $2}' ${tempfile}.score`
    rm -f ${tempfile}.score >& /dev/null

    # create the output pdb
    cat ${tempfile}atoms.pocc |\
    nawk '{x=$1;y=$2;z=$3;occ=$9;B=$5;\
	   Ee=$6;n=$7;chain=$8;\
        printf("%5d%10.5f%10.5f%10.5f%10.5f%5.2f   35%10d%2s   SLX %1s\n",\
    n, x,y,z, B, occ, n, Ee, chain)}' |\
    cat >! ${tempfile}.frac

    # make sure we've got the right cell
    set CELL  = `nawk '$1 == "CELL"{print $3, $4, $5, $6, $7, $8; exit}' ${prefix}.lst`
    coordconv xyzin ${tempfile}.frac xyzout ${tempfile}.pdb << EOF >> $logfile
    CELL $CELL
    INPUT FRAC
    OUTPUT PDB ORTH 1
    END
EOF
    

    # write a descriptive header to the PDB file
    cat << EOF | nawk '{print "REMARK", $0}' >! ${tempfile}full.pdb
results here are from:
$dataset in $mtzfiles at ${cutRES}A in $SG
reflections were excluded if F/sigF < $cutSIG or |delta-F| > $diff
internal score: $score

the B-factor column here contains the "peak height", as reported
by shelx.

the occupancy column contains the result of vector-space refinement of all
the shelx peaks against a Patterson calculated with the same data
provided to shelx.  The CCP4 program vecref will have rejected most of
the atoms, leaving non-zero "occupancy" only for sites that are consistent 
with the Patterson.

ATOMs in the "D" chain are the highest peaks in shelx's direct-phased
Fourier, and represent sites that shelx was "sure" about.

ATOMs in the "Q" chain represent peaks in shelx's direct-phased Fourier
map that may or may not be "real" atoms, but shelx printed them out anyway.

ATOMs in the "X" chain were on special positions.

the symmetry of these positions is $realSG
                           X       Y       Z      occ height
EOF
    # copy over the atoms
    cat ${tempfile}.pdb >> ${tempfile}full.pdb
    rm -f ${tempfile}.pdb >& /dev/null
    rm -f ${tempfile}.frac >& /dev/null
    rm -f ${tempfile}.atoms >& /dev/null


    if($?debug) then
	if(! -e scores.log) touch scores.log
	cat ${tempfile}full.pdb |\
	nawk '/^ATOM/{$0=substr($0, 1, 60) " 20.00"} {print}' |\
	cat >! ${tempfile}test.pdb
	/max/jamesh/Develop/patteq.com $SG o/best_sites.pdb ${tempfile}test.pdb |\
	nawk 'NF==1{printf " %s ", $0; exit}' |\
	cat >> scores.log

	echo "$expect $found $bCFOM $patt $avg $max $sum_s" >> scores.log	
    endif
    

    if($score > $best_score) then
	set best_score = $score
	if(-e ${outfile}.older) mv ${outfile}.older ${outfile}.oldest
	if(-e ${outfile}.old) mv ${outfile}.old ${outfile}.older
	if(-e ${outfile}) mv ${outfile} ${outfile}.old
	mv ${tempfile}full.pdb $outfile >& /dev/null
	if(! $status) echo -n " (saved as $outfile)"
    endif
    
    
    
    # end the line
    echo ""
    
    end	# sites
    end	# diff
    end	# cutSIG
    end	# cutRES
    
end	# dataset
end	# SG

# clean up
Clean_up:
if($?debug) exit
rm -f ${prefix}.hkl >& /dev/null
rm -f ${prefix}.ins >& /dev/null
rm -f ${prefix}.res >& /dev/null
rm -f ${prefix}.lst >& /dev/null
rm -f ${tempfile}atoms.pocc >& /dev/null

rm -f ${tempfile}patt.map >& /dev/null
rm -f ${tempfile}datasets >& /dev/null
rm -f ${tempfile}mtz1.mtz >& /dev/null
rm -f ${tempfile}mtz2.mtz >& /dev/null
rm -f ${tempfile}cadded.mtz >& /dev/null
rm -f ${tempfile}scaled.mtz >& /dev/null
rm -f ${tempfile}scaleit.log >& /dev/null
rm -f ${tempfile}full.pdb >& /dev/null


exit






Setup:
#################################################

  ####   ######   #####  #    #  #####
 #       #          #    #    #  #    #
  ####   #####      #    #    #  #    #
      #  #          #    #    #  #####
 #    #  #          #    #    #  #
  ####   ######     #     ####   #

#################################################
#
#   gather information on:
#    mtz file
#    data sets
#    resolution limits
#    sigma cuttoff (for map generation)
#    difference cutoff
#
##################################################
# scan the command line for files
set user_SGs   = ""
set user_sigs  = ""
set user_resos = ""
set user_diffs = ""
set user_sites = ""
set mtzfiles   = ""
set best_score = -1

foreach arg ( $* )
    # warn about probable mispellings
    if("$arg" =~ *.mtz) then
	if(-e "$arg") then
	    set mtzfiles = "$mtzfiles $arg"
	else
	    echo "WARNING: $arg does not exist! "
	endif
    endif
    if(("$arg" =~ *shelxs)&&(-e "$arg")) then
	# user-specified shelx executable
	set SHELX = "$arg"
    endif
end

# use default if nothing specified
if("$mtzfiles" == "") set mtzfiles = "$mtzfile"

# gather data set labels, etc from the mtzs
set i = 0
echo "" >! ${tempfile}labels
foreach mtz ( $mtzfiles )
    @ i = ( $i + 1 )
    
    # dump and characterize contents
    echo "go" | mtzdump HKLIN $mtz >! ${tempfile}mtzdump

    cat ${tempfile}mtzdump |\
    nawk '/Cell Dimensions/{getline;getline;print "CELL", $0}\
	  /Space group/{print "SYMM", $NF+0}\
	  /Resolution Range/{getline;getline;print "RESO", $4, $6}' |\
    cat >! ${tempfile}info
    
    # read in dataset info
    cat ${tempfile}mtzdump |\
    nawk 'NF>6' |\
    nawk '$(NF-1)=="F"{print "F", $NF, $(NF-4)}\
          $(NF-1)=="D"{print "D", $NF, $(NF-4)}\
          $(NF-1)=="Q"{print "S", $NF, $(NF-4)}' |\
    nawk '/^F/{++n} /^D/{++n} {printf "%s", $1; \
           if($1=="S") printf "%s", last;\
    printf " %d %s %s\n",n, $2, $3; last=$1}' |\
    cat >> ${tempfile}info
    
    # check for empty mtzs
    set temp = `nawk '/^[FD]/' ${tempfile}info | wc -l`
    if($temp > 0) then
	# add it to the pile
	cat ${tempfile}info |\
	nawk -v mtz=$mtz '{print $0, mtz}' |\
	cat >> ${tempfile}labels
    else
	echo "WARNING: $mtz contains no useful data!"
    endif
    if(! $?debug) rm -f ${tempfile}mtzdump >& /dev/null
    if(! $?debug) rm -f ${tempfile}info >& /dev/null
end

# get complete, unique lists of DANOs
cat ${tempfile}labels |\
nawk '{l= $2 " " $NF}\
      $1=="D"{D[l]=$3; signal[l]=$4} \
      $1=="SD"{S[l]=$3; noise[l]=$4} \
      END{for(l in S){if(noise[l]+0==0) noise[i]=1; \
	print "D", D[l], S[l], signal[l]/noise[l], l}}' |\
nawk 'NF==6' |\
sort -nr +2 >! ${tempfile}mtzlabels

# get complete, unique lists of Fs
cat ${tempfile}labels |\
nawk '{l= $2 " " $NF}\
      $1=="F"{F[l]=$3; signal[l]=$4} \
      $1=="SF"{S[l]=$3; noise[l]=$4} \
      END{for(l in S){if(noise[l]+0==0) noise[l]=1; \
	print "F", F[l], S[l], signal[l]/noise[l], l}}' |\
nawk 'NF==6' |\
sort -nr +2 >> ${tempfile}mtzlabels

# reconcile different cells? SGs?
set CELL  = `nawk '/^CELL/{print $2, $3, $4, $5, $6, $7; exit}' ${tempfile}labels`
set SG    = `nawk '/^SYMM/{print $2; exit}' ${tempfile}labels`
set SGnum = `nawk '/^SYMM/{print $2; exit}' ${tempfile}labels`
set SG    = ` nawk -v num=$SGnum '$1==num && NF>5{print $4}' ${CLIBD}/symop.lib `
set hiRES = `nawk '/^RESO/{print $3}' ${tempfile}labels | sort -n | head -1`

if(! $?debug) rm -f ${tempfile}labels >& /dev/null


set mtzlabels = `cat ${tempfile}mtzlabels | wc -l`
if("$mtzlabels" == 0) then
    echo "no usefult data in $mtzfiles "
    goto Help
endif

##################################################
# get crystal and dataset information from the final mtz file


# make a two-way difference-dataset list (for recognition)
cat ${tempfile}mtzlabels |\
nawk '$1=="F"{++n; set[n]= $2 " " $3 " " $NF;\
      for(i=1;i<n;++i){print "F", set[i], set[n]; print "F", set[n], set[i];}}' |\
cat >! ${tempfile}Fpairs


# one last pass through command line
# allow user overrides of all internal variables
set i = 0
echo -n "" >! ${tempfile}userlabels
echo -n "" >! ${tempfile}userpairs
echo -n "" >! ${tempfile}not_pairs
while( $i < $#argv )
    @ i = ( $i + 1 )
    @ nexti = ( $i + 1 )
    @ lasti = ( $i - 1 )
    if($nexti > $#argv) set nexti = $#argv
    if($lasti < 1) set lasti = 1
    set arg = "$argv[$i]"
    
    # check for space groups
    if("$arg" =~ [PpCcIiFfRrHh][1-6]*) then
	set temp = `echo $arg | nawk '{print toupper($1)}'`
	set temp = `nawk -v SG=$temp '$4 == SG {print $4}' $CLIBD/symop.lib | head -1`
	if("$temp" == "") then
	    # check for "pseudo-spacegroup" language
	    set temp = `echo "$arg" | nawk 'toupper($1) ~ /[PC]2212|[PC]2122|P21221|P22121/'`
	    # these are okay too, reindexing engine will understand
	endif
	if("$temp" != "") then
	    # add this SG to the space group list
	    set user_SGs = "$user_SGs $temp"
	    continue
	endif
    endif
    
    # only look at non-file words
    if(! -e "$arg") then
	# see if a dataset label was given
	set temp = `nawk -v arg=$arg '$2==arg' ${tempfile}mtzlabels | wc -l`
	if($temp) then
	    # a dataset label was mentioned
	    if($?NO) then
		# user doesn't want this label
		# filter it out of the input files
		nawk -v arg=$arg '$2!=arg' ${tempfile}mtzlabels >! ${tempfile}
		mv ${tempfile} ${tempfile}mtzlabels
	    else
		# must want only this label?
		nawk -v arg=$arg '$2==arg' ${tempfile}mtzlabels |\
		cat >> ${tempfile}userlabels
	    endif
	    continue
	endif
    	
	# see if a difference-dataset label was given
	set temp = `nawk -v arg=$arg '$2 "-" $5 == arg{print $2, $5; exit}' ${tempfile}Fpairs`
	if($#temp == 2) then
	    # a specific difference dataset was mentioned
	    if($?NO) then
		# user doesn't want this label pair
		# filter it out of the input files
		echo "NOT_A_PAIR $temp" >> ${tempfile}not_pairs
	    else
		# must want only this pair (in this order)?
		cat ${tempfile}Fpairs |\
		nawk -v F1=$temp[1] -v F2=$temp[2] '$2==F1 && $5==F2' |\
		cat >> ${tempfile}userpairs
	    endif
	    continue	    
	endif
	
	# program options
	if("$arg" =~ [Pp]att*) then
	    if($?NO) then
		set NO_PATT
	    else
		unset NO_PATT
	    endif
	endif
	
	# check for numbers
	if("$arg" =~ [0-9]*) then
	    # we have a number
	    if(("$arg" =~ *A)||("$argv[$nexti]" == "A")) then
		# user-preferred resolution limits
		set temp = `echo "$arg" | nawk 'BEGIN{FS="-"} $1+0 > 0.1{print $1+0} $2+0 > 0.1{print $2+0}'`
		if($#temp != 1) then
		    set temp = `echo $temp | nawk '$1>$2{print $1, $2} $2>$1{print $2, $1}'`
		    if($#temp == 2) then
				set loRES = "$temp[1]"
				set user_resos = "$user_resos $temp[2]"
				continue
		    endif
		else
		    if("$temp" != "") set user_resos = "$user_resos $temp"
		    continue
		endif
	    endif
	    
	    if(("$arg" =~ *[0-9][Ss]*)||("$argv[$nexti]" =~ [Ss]igma)) then
		set user_sigs = "$user_sigs "`echo "$arg" | nawk '$1+0 > 0{print $1+0}'`
	    endif
	    
	    if(("$argv[$nexti]" =~ [Ss]ite)||("$argv[$nexti]" =~ [Ss]ites)) then
		set user_sites = "$user_sites "`echo "$arg" | nawk '$1+0 > 0{print $1+0}'`
	    endif
	    
	    if(("$arg" =~ *[Dd]iff)||("$argv[$lasti]" =~ [Dd]iff)) then
		set user_diffs = "$user_diffs "`echo "$arg" | nawk '$1+0 > 0{print $1+0}'`
	    endif
	    if(("$arg" =~ *[Dd]iso)||("$argv[$lasti]" =~ [Dd]iso)) then
		set MAX_diso = `echo "$arg" | nawk '$1+0 > 0{print $1+0}'`
	    endif
	    if(("$arg" =~ *[Dd]ano)||("$argv[$lasti]" =~ [Dd]ano)) then
		set MAX_dano = `echo "$arg" | nawk '$1+0 > 0{print $1+0}'`
	    endif
	endif
	
	# allow "NO" logic to carry through
	unset NO
	if(("$arg" == "no")||("$arg" == "not")) set NO
	if(("$arg" == "don't")||("$arg" == "ignore")) set NO
	if("$arg" == "except") set NO
    endif
end

# honor user requests (when mentioned)
if("$user_SGs"   != "") set SGs           = "$user_SGs"
if("$user_resos" != "") set RESOs         = "$user_resos"
if("$user_sigs"  != "") set sigma_cutoffs = "$user_sigs"
if("$user_diffs" != "") set max_diffs     = "$user_diffs"
if("$user_sites" != "") set target_sites  = "$user_sites"


# check the "user" labels
sort -u +5 ${tempfile}userlabels >! ${tempfile}mtzs
sort -u +5 ${tempfile}mtzlabels >> ${tempfile}mtzs
cat ${tempfile}mtzs |\
nawk '{++seen[$6]} END{for(mtz in seen) if(seen[mtz]==1) print mtz}' |\
cat >! ${tempfile}orphans

foreach mtz ( `cat ${tempfile}orphans` )
    # some mtzs were mentioned, but aren't in ${tempfile}userlabels
    # so, we'll just assume everything in them was wanted
    cat ${tempfile}mtzlabels |\
    nawk -v mtz=$mtz '$6==mtz' |\
    cat >> ${tempfile}userlabels
end
if(! $?debug) rm -f ${tempfile}orphans   >& /dev/null
if(! $?debug) rm -f ${tempfile}mtzlabels >& /dev/null
if(! $?debug) rm -f ${tempfile}mtzs      >& /dev/null

# make sure no label is mentioned twice
cat ${tempfile}userlabels |\
nawk '! seen[$2 " " $NF]{print} {++seen[$2 " " $NF]}' |\
cat >! ${tempfile}
mv ${tempfile} ${tempfile}userlabels

# organize user's desired labels into difference-data sets
cat ${tempfile}userlabels |\
nawk '$1=="F"{++n; set[n]= $2 " " $3 " " $NF;\
      for(i=1;i<n;++i) print "F", set[i], set[n]}' |\
cat >! ${tempfile}Fpairs

# see if user had particular pairing in mind
set temp = `cat ${tempfile}userpairs | wc -l`
if($temp) then
    # this overrides the Fpairs we just generated
    cat ${tempfile}userpairs >! ${tempfile}Fpairs
    
    # also any user-specified individual labels must be lone Fs? 
    cat ${tempfile}userlabels |\
    nawk '$1=="F"{print "F", $2, $3, $NF}' |\
    cat >> ${tempfile}Fpairs
endif
if(! $?debug) rm -f ${tempfile}userpairs >& /dev/null

set Fpairs = `nawk '$1=="F"' ${tempfile}Fpairs | wc -l`
if(! $Fpairs) then
    # maybe there's just one, lone F?
    cat ${tempfile}userlabels |\
    nawk '$1=="F"{print "F", $2, $3, $NF}' |\
    cat >> ${tempfile}Fpairs
endif


# now filter out user-unwanted pairings
set temp = `cat ${tempfile}not_pairs | wc -l`
if($temp) then
    # user didn't want particular pairings
    cat ${tempfile}not_pairs ${tempfile}Fpairs |\
    nawk '/^NOT_A_PAIR/{not_a_pair[$2 " " $3]=1; not_a_pair[$3 " " $2]=1;}\
        ! /^NOT_A_PAIR/{if(! not_a_pair[$2 " " $5]) print}' |\
    cat >! ${tempfile}
    mv ${tempfile} ${tempfile}Fpairs
endif

# assemble Diso and Dano data into the same "dataset" file
cat ${tempfile}userlabels |\
nawk '$1=="D"{print "D", $2, $3, $NF}' |\
cat >! ${tempfile}datasets
cat ${tempfile}Fpairs >> ${tempfile}datasets

if(! $?debug) rm -f ${tempfile}Fpairs     >& /dev/null
if(! $?debug) rm -f ${tempfile}userlabels >& /dev/null
if(! $?debug) rm -f ${tempfile}userpairs  >& /dev/null
if(! $?debug) rm -f ${tempfile}not_pairs  >& /dev/null

goto Return_from_Setup

####################################################

The Future:
mtz or hkl input
support intensity data
